from datetime import datetime
import lxml
from lxml import html
import requests
import numpy as np
import pandas as pd
import bs4

""" 

    To get the name of symbol/ticker of the stocks for which you want information you can
    look it up on https://finance.yahoo.com/ and from there you can pass the scrip name
    in the parameter where required. 

"""

headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'en-US,en;q=0.9',
    'Cache-Control': 'max-age=0',
    'Pragma': 'no-cache',
    'Referrer': 'https://google.com',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36'
}

def _calculate_financials(symbol, statement):

    """

        calculate_financials function is used to fetch and preprocess the financial documents
        from the Yahoo Finance website.

        Parameters
        --------------------------------

        symbol :    It is used to specify the symbol/ticker for which the 
                    financials have to be fetched

        statement : The name of the financial statement that has to be fetched.
                    Categories = [
                                  'balance-sheet' : Fetches the balance sheet,
                                  'cash-flow' :     Fetches the cash flow statement,
                                  'financials' :    Fetches the income statement
                                 ]

        Returns
        --------------------------------
        
        A dataframe that contains the required financial statement.

    """

    headers = []
    temp_list = []
    label_list = []
    final = []
    index = 0
    url = 'https://finance.yahoo.com/quote/' + symbol + '/' + statement + '?p=' + symbol
    page = requests.get(url, headers)
    soup = bs4.BeautifulSoup(page.content, 'html.parser')
    features = soup.findAll('div', class_='D(tbr)')
    for item in features[0].find_all('div', class_='D(ib)'):
        headers.append(item.text)
    while index <= len(features) - 1:
        temp = features[index].findAll('div', class_='D(tbc)')
        for line in temp:
            temp_list.append(line.text)
        final.append(temp_list)
        temp_list = []
        index += 1

    df = pd.DataFrame(final[1:])
    df.columns = headers

    return df

def get_balance_sheet(symbol):

    """ 

        Used to obtain the balance sheet of the specified ticker

        Parameters
        --------------------------------

        symbol :    It is used to specify the symbol/ticker for which the 
                    balance sheet has to be fetched

        Returns
        --------------------------------
        
        A dataframe that contains the balance sheet of the company
    
    """

    bal_sheet = _calculate_financials(symbol, 'balance-sheet')
    return bal_sheet

def get_cash_flow(symbol):

    """ 

        Used to obtain the cash flow statement of the specified ticker

        Parameters
        --------------------------------

        symbol :    It is used to specify the symbol/ticker for which the 
                    cash flow has to be fetched

        Returns
        --------------------------------
        
        A dataframe that contains the cash flow statement of the company
    
    """

    cash_flow = _calculate_financials(symbol, 'cash-flow')
    return cash_flow

def get_income_statement(symbol):

    """ 

        Used to obtain the income statement of the specified ticker

        Parameters
        --------------------------------

        symbol :    It is used to specify the symbol/ticker for which the 
                    income statement has to be fetched

        Returns
        --------------------------------
        
        A dataframe that contains the income statement of the company
    
    """

    inc_statement = _calculate_financials(symbol, 'financials')
    return inc_statement




import constants
import pandas as pd
import yfinance as web
import datetime
import numpy as np

TODAY = datetime.date.today()
PREV = datetime.timedelta(90)

#modify this in new feature
# def get_nifty():
#     ticker = pd.read_html(constants.BASE_URL)[2]
#     print(ticker)
#     return ticker

def get_closing_price(tickers, start = TODAY - PREV, end = TODAY):

    """ 

        This function returns the Closing price of a list of tickers mentioned in the parameter.
        Functions that are defined above can be directly passed to get the closing price of 
        each ticker in the index

        Parameters
        --------------------------------

        tickers : Contains a list of symbols for which the closing price will be returned

        start :   Contains the starting date from which closing price is required
                  Format: 'dd/mm/yyyy' as in '25/04/2021' 
                  Default: Three months from today's date

        end :     Contains the end date till which closing price is required
                  Format: 'dd/mm/yyyy' as in '27/04/2021' 
                  Default: Today's date

        Other than a custom list some other parameters that can be passed directly are:

        - get_sensex()
        - get_nifty_next50()
        - get_nifty_bank()
        - get_nifty_auto()
        - get_nifty_financial()
        - get_nifty_fmcg()
        - get_nifty_it()
        - get_nifty_media()
        - get_nifty_metal()
        - get_nifty_pharma()
        - get_nifty_psubank()
        - get_nifty_privatebank()
        - get_nifty_realty()
        - get_nifty()

        Returns
        --------------------------------

        A pandas dataframe that contains the closing price of all symbols passed as the 
        parameter to the function.

    """
    
    Closing = pd.DataFrame()
    for i in range(len(tickers)):
      try:
        temp = web.download(tickers[i], start = start, end = end)
        temp.dropna(inplace=True)
        Closing[tickers[i]] = temp['Close']
      except:
        print("No info is available for this particular stock " + tickers[i])
    return Closing

def get_data(ticker, start = TODAY - PREV, end = TODAY):

    """ 

        This function returns the various attributes of a ticker such as the High, Low, 
        Open, Close, Volume and Adjusted Close 

        Parameters
        --------------------------------

        tickers : Contains the symbol/ticker for which various attributes will be returned

        start :   Contains the starting date
                  Format: 'dd/mm/yyyy' as in '25/04/2021' 
                  Default: Three months from today's date

        end :     Contains the end date
                  Format: 'dd/mm/yyyy' as in '27/04/2021' 
                  Default: Today's date

        Returns
        --------------------------------

        A pandas dataframe that contains various attributes of a ticker such as the High, Low, 
        Open, Close, Volume and Adjusted Close 

    """

    temp = web.download(ticker, start=start, end=end)
    return temp

def get_live_price(ticker):

    """ 

        This function returns the live/latest price for the symbol that has been passed as the
        parameter.

        Parameters
        --------------------------------
        ticker : Contains the symbol/ticker for which the live price will be returned

    """

    temp = web.download(ticker, TODAY - PREV)['Close']
    return np.round(temp[-1],2)

def get_summary(symbol):

    """ 

        This function returns the summary of various attributes of the symbol/ticker that 
        has been passed as the parameter.

        Parameters
        --------------------------------

        tickers : Contains the symbol/ticker for which the summary of various attributes
                  will be returned

        Returns
        --------------------------------

        A pandas dataframe that contains various attributes of a ticker such as the:
        - Previous Close
        - Open
        - Bid
        - Ask
        - Day's Range
        - 52 Week Range
        - Volume
        - Average Volume
        - Market Cap
        - Beta
        - P/E Ratio
        - EPS
        - Earnings Date
        - Forward Dividend and Yield
        - Ex-Dividend Date
        - 1 Year Target Estimate
             
    """
    
    link = pd.read_html('https://finance.yahoo.com/quote/' + symbol + '?p=' + symbol)
    link1 = pd.concat([link[0], link[1]],ignore_index=True)
    link1.columns = ['Attribute', 'Value']
    return link1


def get_crypto_data(symbol):

    """ 

        This function returns the various attributes of a crypto ticker such as the High, Low, 
        Open, Close, Volume and Adjusted Close.
        This is limited to the previous 100 days historical data for the coin

        Parameters
        --------------------------------

        symbol : Contains the symbol/ticker for which various attributes will be returned
        Example: crypto = get_crypto_data('BTC-USD)

        Returns
        --------------------------------

        A pandas dataframe that contains various attributes of a crypto coin such as the High, Low, 
        Open, Close, Volume and Adjusted Close 

    """

    table1 = pd.read_html('https://finance.yahoo.com/quote/' + symbol + '/history?period1=1410912000&period2=1622678400&interval=1d&filter=history&frequency=1d&includeAdjustedClose=true')
    crypto_data = table1[0][:-1]
    crypto_data['Date']= pd.to_datetime(crypto_data['Date'])
    crypto_data = crypto_data.set_index('Date')
    return crypto_data



import numpy as np
import pandas as pd
import plotly.express as px
import matplotlib.pyplot as plt
from plotly.offline import init_notebook_mode, iplot
import plotly.graph_objs as go
import plotly.figure_factory as ff
import datetime
from urllib.parse import urlencode
import yfinance as web

TODAY = datetime.date.today()
PREV = datetime.timedelta(600)


def moving_avg(scrip, num_days):

    """ 
        Parameters
        -------------------------------
        scrip : Used to specify the symbol/ticker for which the moving average has to be plotted
        num_days : Number of days for which moving average has to be plotted. Commonly used values
                    are 14, 20, 50, 100, 200 
        
        Returns
        --------------------------------
        Plot consisting of moving average along with the closing price
    """

    company = web.download(scrip, start = TODAY-PREV, end = TODAY)
    company['MA-' + str(num_days)] = company['Close'].rolling(num_days).mean()
    fig = go.Figure()
    fig.add_trace(go.Scatter(x = company.index, y = company['MA-' + str(num_days)], name = str(num_days) + "Day MA",line = dict(color='orange', width = 1.2)))
    fig.add_trace(go.Scatter(x = company.index, y = company['Close'], name = "Closing Price",line = dict(color='green', width = 1.2)))
    fig.update_xaxes(
        rangeslider_visible=True)
    fig.show()


def bollinger_bands(scrip):
    """ 
        Parameters
        -------------------------------
        scrip : Used to specify the symbol/ticker for which Bollinger Bands has to be plotted 
        
        Returns
        --------------------------------
        Plot consisting of Bollinger Bands for ticker
    """

    company = web.download(scrip, start = TODAY-PREV, end = TODAY)
    company['20 day Close MA'] = company['Close'].rolling(20).mean()
    company['Upper'] = company['20 day Close MA'] + (2 * company['Close'].rolling(20).std())
    company['Lower'] = company['20 day Close MA'] - (2 * company['Close'].rolling(20).std())

    fig = go.Figure()
    fig.add_trace(go.Scatter(x = company.index, y = company['20 day Close MA'],
                  name = "20 Day MA",line = dict(color='rgb(117, 112, 179)', width = 1.2)))
    fig.add_trace(go.Scatter(x = company.index, y = company['Upper'],
                             name = "Upper Limit",line = dict(color='rgb(166, 86, 40)', width = 1.2)))
    fig.add_trace(go.Scatter(x = company.index, y = company['Lower'],
                             name = "Lower Limit",line = dict(color='#FD3216', width = 1.2)))
    fig.add_trace(go.Scatter(x = company.index, y = company['Close'],
                             name = "Closing Price",line = dict(color='rgb(27, 158, 119)', width = 1.2)))#'#19D3F3'
    fig.show()

def get_chart(scrip, kind = 'line',start = TODAY-PREV, end = TODAY):

    """ 
        Parameters
        -------------------------------
        scrip : Used to specify the symbol/ticker for which historical chart has to be plotted

        kind : The type of chart - 'line' or 'area'
        
        start :   Contains the starting date
                  Format: 'dd/mm/yyyy' as in '25/04/2020' 
                  Default: 600 days from today's date

        end :     Contains the end date
                  Format: 'dd/mm/yyyy' as in '27/05/2021' 
                  Default: Today's date

        Returns
        --------------------------------
        Historical chart based on time frame
    """

    company = web.download(scrip, start = start, end = end)
    if kind.lower() == 'line':
        fig = px.line(company, y = 'Close', x = company.index,range_x=[start,end])
        fig.update_xaxes(rangeslider_visible=True)
        fig.show()
    elif kind.lower() == 'area':
        fig = px.area(company, y = 'Close', x = company.index,range_x=[start,end])
        fig.update_xaxes(rangeslider_visible=True)
        fig.show()


# TO-DO
"""
Incorporate weighted and exponential moving average for the moving average function
"""

"""UPDATE FOR THE NEXT VERSION
    ADD timestamps such as 1m, 2m, 6m, ytd in Bollinger bands
"""


from setuptools import setup
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='nifpy',
    version = '1.1.0',
    author = 'Shresth Singh',
    author_email = 'singhshresth26@gmail.com',
    description = 
                  """
                        Easy to use python package that can be used to fetch live price, closing price, stock summary, 
                        index list and fundamentals such as income statement, cash flow statement
                        and balance sheet for stocks that trade on the National Stock Exchange(NSE).
                        Package has been updated and now supports crypto currency.
                  """,
    long_description=long_description,
    long_description_content_type='text/markdown',
    url = 'https://github.com/shresth26/nifpy',
    license = 'MIT',
    packages = ['nifpy'],
    keywords = ['stocks', 'nifty', 'financials','algo trading','nse','crypto','cryptocurrency','sensex'],
zip_safe = False)
